use crate::{definition::Scope, server::LSPServer, sources::LSPSupport, utils::{get_definition_token, get_language_id_by_uri}};
use sv_parser::{RefNode, SyntaxTree};
use crate::definition::extract_defs::get_ident;
use tower_lsp::lsp_types::*;

impl LSPServer {
    pub fn document_highlight(
        &self,
        params: DocumentHighlightParams,
    ) -> Option<Vec<DocumentHighlight>> {
        let uri = params.text_document_position_params.text_document.uri;
        let pos = params.text_document_position_params.position;
        let file_id = self.srcs.get_id(&uri).to_owned();
        self.srcs.wait_parse_ready(file_id, false);
        let file = self.srcs.get_file(file_id)?;
        let file = file.read().ok()?;

        let line_text = file.text.line(pos.line as usize);
        let token = get_definition_token(&line_text, pos);

        let language_id = get_language_id_by_uri(&uri);
        
        let scope_tree = self.srcs.scope_tree.read().ok()?;
        // use the byte_idx of the definition if possible, otherwise use the cursor
        let byte_idx =
            match scope_tree
                .as_ref()?
                .get_definition(&token, file.text.pos_to_byte(&pos), &uri)
            {
                Some(def) => def.byte_idx,
                None => file.text.pos_to_byte(&pos),
            };
        let syntax_tree = file.syntax_tree.as_ref()?;
        let references = all_identifiers(syntax_tree, &token);
        
        Some(
            scope_tree
                .as_ref()?
                .document_highlights(&uri, &file.text, references, byte_idx),
        )
    }
}

/// return all identifiers in a syntax tree matching a given token
fn all_identifiers(syntax_tree: &SyntaxTree, token: &str) -> Vec<(String, usize)> {
    let mut idents: Vec<(String, usize)> = Vec::new();
    for node in syntax_tree {
        if let RefNode::Identifier(_) = node {
            
            let (ident, byte_idx) = get_ident(syntax_tree, node);
            if ident == token {
                idents.push((ident, byte_idx));
            }
        }
    }
    idents
}